#!/usr/bin/perl -w
#
#  perl plot.pl < movementXXX.txt > movementXXX.xpm
#
#  Converts a move.pas output into an XPM picture.
#  Also adds some useful comments to the XPM.
#
use strict;

my @exactPoints;
my @inexactPoints;

while (<>) {
    s/^\s+//;
    if (/^-?\d/) {
        my ($orderX, $orderY, $moveX, $moveY, $remainX, $remainY, @rest)
          = split /\s+/;
        my $data = { ox => $orderX, oy => $orderY,
                     mx => $moveX,  my => $moveY,
                     od => $orderX*$orderX + $orderY*$orderY,
                     md => $moveX*$moveX + $moveY*$moveY };

        if ($remainX == 0 && $remainY == 0) {
            # This point is reached
            push @exactPoints, $data;
        } else {
            # Point is not reached
            push @inexactPoints, $data;
        }
    }
}

# Find minimum/maximum inexact distance
my $minInexactDist;
my $maxInexactDist;
my $maxTravelDist;
my $maxTravelPoint;
foreach (@inexactPoints) {
    if (!defined($minInexactDist) || $_->{od} < $minInexactDist) {
        $minInexactDist = $_->{od};
    }
    if (!defined($maxInexactDist) || $_->{od} > $maxInexactDist) {
        $maxInexactDist = $_->{od};
    }
    if (!defined($maxTravelDist) || $_->{md} > $maxTravelDist) {
        $maxTravelDist = $_->{md};
        $maxTravelPoint = $_;
    }
}
print "/* Minimum inexact distance: ", sqrt($minInexactDist), " */\n";
print "/* Maximum inexact distance: ", sqrt($maxInexactDist), " */\n";

my $minExactDist;
my $maxExactDist;
my $maxExactPoint;
my $maxExactSafe;
foreach (@exactPoints) {
    if (!defined($minExactDist) || $_->{od} < $minExactDist) {
        $minExactDist = $_->{od};
    }
    if (!defined($maxExactDist) || $_->{od} > $maxExactDist) {
        $maxExactDist = $_->{od};
        $maxExactPoint = $_;
    }
    if (!defined($maxTravelDist) || $_->{md} > $maxTravelDist) {
        $maxTravelDist = $_->{md};
        $maxTravelPoint = $_;
    }
    if ($_->{md} < $minInexactDist && (!defined($maxExactSafe) || $_->{md} > $maxExactSafe)) {
        $maxExactSafe = $_->{md}
    }
}
print "/* Minimum exact distance: ", sqrt($minExactDist), " */\n";
print "/* Maximum exact distance: ", sqrt($maxExactDist), " $maxExactPoint->{mx},$maxExactPoint->{my} */\n";
print "/* Minimum safe distance: ", sqrt($maxExactSafe), " */\n";

print "/* Maximum travel distance: ", sqrt($maxTravelDist), " $maxTravelPoint->{mx},$maxTravelPoint->{my} ",
  "with order $maxTravelPoint->{ox},$maxTravelPoint->{oy} */\n";

my %image;
my %inexactTargets;
foreach (@inexactPoints) {
    # Mark that this point is reached
    $image{$_->{my}}{$_->{mx}} = 'x';
    if (!exists $inexactTargets{"$_->{my},$_->{mx}"}
        || $_->{od} < $inexactTargets{"$_->{my},$_->{mx}"}{od})
      {
          $inexactTargets{"$_->{my},$_->{mx}"} = $_;
      }

    if ($_->{md} == $maxTravelDist) {
        $image{$_->{my}}{$_->{mx}} = 'X';
        $inexactTargets{"$_->{my},$_->{mx}"} = $_;
    }
}

foreach (@exactPoints) {
    if ($_->{od} < $minInexactDist) {
        # Point is always reached
        $image{$_->{my}}{$_->{mx}} = 'r';
    } else {
        # Point is reached but other points of the same distance are not
        $image{$_->{my}}{$_->{mx}} = 'R';
        printf "/* Outside safe distance: %d,%d  (%.2f ly) */\n",
          $_->{mx}, $_->{my}, sqrt($_->{md})
            if $_->{my} >= 0 && $_->{mx} >= 0;
    }

    if ($_->{md} == $maxTravelDist) {
        $image{$_->{my}}{$_->{mx}} = 'Y';
    }

    delete $inexactTargets{"$_->{my},$_->{mx}"};
}

foreach (values %inexactTargets) {
    printf "/* Inexact: %d,%d  (%.2f)  moves %d,%d (%.2f) */\n",
      $_->{ox}, $_->{oy}, sqrt($_->{od}), $_->{mx}, $_->{my}, sqrt($_->{md})
        if ($_->{ox} >= 0 && $_->{oy} >= 0);
    $image{$_->{oy}}{$_->{ox}} = 'G';
}


# Print it
my $minX = 0;
my $maxX = 0;
my $minY = 0;
my $maxY = 0;
my $symm = 1;
foreach my $y (keys %image) {
    $minY = $y if $y < $minY;
    $maxY = $y if $y > $maxY;
    foreach my $x (keys %{$image{$y}}) {
        $minX = $x if $x < $minX;
        $maxX = $x if $x > $maxX;
        if (   ($image{$y}{-int($x)}  || 0) ne $image{$y}{$x}
            || ($image{-int($y)}{$x}  || 0) ne $image{$y}{$x}
            || ($image{-int($y)}{-int($x)} || 0) ne $image{$y}{$x})
          {
              $symm = 0;
              printf "/* Asymmetry at $x,$y */\n";
              $image{$y}{$x} = 'A';
          }
    }
}

if ($symm) {
    # Symmetric? Cut negative halves
    foreach my $y (keys %image) {
        if ($y > 0) {
            delete $image{$y}
        } else {
            foreach my $x (keys %{$image{$y}}) {
                if ($x < 0) {
                    delete $image{$y}{$x}
                }
            }
        }
    }
    $minX = $maxY = 0;
}

print "/* X = ($minX, $maxX), Y = ($minY, $maxY) */\n";

print "static char *test[] = {\n";
printf "\"%d %d %d %d\",\n", 3*($maxX-$minX+1), 3*($maxY-$minY+1), 8, 1;
print "\". c #000000\",\n";
print "\", c #444444\",\n";
print "\"X c #ff00ff\",\n";
print "\"x c #ff0000\",\n";
print "\"R c #00ff00\",\n";
print "\"r c #008800\",\n";
print "\"Y c #ffff00\",\n";
print "\"G c #0000ff\",\n";

foreach my $y ($minY .. $maxY) {
    foreach (0 .. 1) {
        print "\"";
        foreach my $x ($minX .. $maxX) {
            print (($image{$y}{$x} || '.') x 2, '.');
        }
        print "\",\n";
    }
    print "\"";
    foreach my $x ($minX .. $maxX) {
        if ($x % 10 || $y % 10) {
            print "...";
        } else {
            print "..,";
        }
    }
    print "\",\n";
}
print "};\n";
